#include "wall_construction_qt.h"
#include <argos3/core/wrappers/lua/lua_controller.h>
#include <argos3/plugins/robots/foot-bot/simulator/footbot_entity.h>

/****************************************/
/****************************************/
 
CWallConstructionQT::CWallConstructionQT() {
   RegisterUserFunction<CWallConstructionQT,CFootBotEntity>(&CWallConstructionQT::Draw);
}
 
/****************************************/
/****************************************/
 
void CWallConstructionQT::Draw(CFootBotEntity& c_entity) {
   /* Get reference to robot controller */
   CLuaController& cContr = dynamic_cast<CLuaController&>(c_entity.GetControllableEntity().GetController());
   /* This is the message that will be shown */
   std::string strMsg;
   /* This is the list of rays that will be drawn */
   std::vector<CVector2> cRayCoords;
   std::vector<CColor> cRayColors;
   /* Get 'robot' table */
   lua_getglobal(cContr.GetLuaState(), "robot");
   if(! lua_isnil(cContr.GetLuaState(), -1)) {
      /* Get 'message' variable */
      lua_getfield(cContr.GetLuaState(), -1, "message");
      if((! lua_isnil(cContr.GetLuaState(), -1)) &&              // 'message' variable found
         (lua_isstring(cContr.GetLuaState(), -1))) {             // its value is a string
         /* Get value as a real number */
         strMsg += lua_tostring(cContr.GetLuaState(), -1);
      }
      /* Clean up Lua stack */
      lua_pop(cContr.GetLuaState(), 1);
      /* Get the 'vectors' table */
      lua_getfield(cContr.GetLuaState(), -1, "vectors");
      if((! lua_isnil(cContr.GetLuaState(), -1)) &&             // 'vectors' variable found
         (lua_istable(cContr.GetLuaState(), -1))) {             // its value is a table
         /* Go through the table elements */
         lua_pushnil(cContr.GetLuaState());
         while(lua_next(cContr.GetLuaState(), -2)) {
            /* Is this element a table? */
            if(lua_istable(cContr.GetLuaState(), -1)) {
               CVector2 cCoords;
               CColor cColor;
               lua_getfield(cContr.GetLuaState(), -1, "x");
               if(!lua_isnil(cContr.GetLuaState(), -1) &&
                  lua_type(cContr.GetLuaState(), -1) == LUA_TNUMBER) {
                  cCoords.SetX(lua_tonumber(cContr.GetLuaState(), -1) * 100.0);
               }
               lua_pop(cContr.GetLuaState(), 1);
               lua_getfield(cContr.GetLuaState(), -1, "y");
               if(!lua_isnil(cContr.GetLuaState(), -1) &&
                  lua_type(cContr.GetLuaState(), -1) == LUA_TNUMBER) {
                  cCoords.SetY(lua_tonumber(cContr.GetLuaState(), -1) * 100.0);
               }
               lua_pop(cContr.GetLuaState(), 1);
               lua_getfield(cContr.GetLuaState(), -1, "color");
               if(!lua_isnil(cContr.GetLuaState(), -1) &&
                  lua_type(cContr.GetLuaState(), -1) == LUA_TSTRING) {
                  cColor.Set(lua_tostring(cContr.GetLuaState(), -1));
               }
               lua_pop(cContr.GetLuaState(), 1);
               /* Add item to lists */
               cRayCoords.push_back(cCoords);
               cRayColors.push_back(cColor);
            }
            /* Done with element */
            lua_pop(cContr.GetLuaState(), 1);
         }
      }
      /* Clean up Lua stack */
      lua_pop(cContr.GetLuaState(), 2);
   }
   /* Disable face culling to be sure the text is visible from anywhere */
   glDisable(GL_CULL_FACE);
   /* Draw rays */
   for(size_t i = 0; i < cRayCoords.size(); ++i) {
      DrawRay(CRay3(CVector3(0.0, 0.0, 0.3),
                    CVector3(cRayCoords[i].GetX(),
                             cRayCoords[i].GetY(),
                             0.3)),
              cRayColors[i]);
   }
   /* Disable lighting, so it does not interfere with the chosen text color */
   glDisable(GL_LIGHTING);
   /* Set the text color */
   CColor cColor(CColor::BLACK);
   glColor3ub(cColor.GetRed(), cColor.GetGreen(), cColor.GetBlue());
   /* The position of the text is expressed wrt the reference point of the footbot
    * For a foot-bot, the reference point is the center of its base.
    * See also the description in
    * $ argos3 -q foot-bot
    */
   DrawText(CVector3(0.0, 0.0, 0.4), // position
            strMsg);                 // text
   /* Restore lighting */
   glEnable(GL_LIGHTING);
   /* Restore face culling */
   glEnable(GL_CULL_FACE);
}
 
/****************************************/
/****************************************/
 
REGISTER_QTOPENGL_USER_FUNCTIONS(CWallConstructionQT, "id_qtuser_functions")
